////////////////////////////////////////////////////////////////////////////////
// File         : HnetUniversePrint.cpp
// Subsystem    : Graph
// Component    : H-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class HnetUniversePrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 19.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
//     
////////////////////////////////////////////////////////////////////////////////

#include "HnetUniversePrint.h"
#include "HnetPrint.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

HnetUniversePrint::HnetUniversePrint( const char *name ) 
	: ObjPrint( "HnetUniverse", name ), NodesNumbers(), HnetUniverse()
{
	Renumber();  // for Root
}


HnetUniversePrint::HnetUniversePrint( HnetUniversePrint &universe, const char *name ) : ObjPrint( "HnetUniverse", name )
{
	Copy( universe );
};


HnetUniversePrint::HnetUniversePrint( const HnetUniverse &universe, const char *name ) : ObjPrint( "HnetUniverse", name )
{
	HnetUniverse::Copy( universe );
	Renumber();
};


HnetUniversePrint::~HnetUniversePrint()
{

}


//////////////////////////////////////////////////////////////////////
// public
//////////////////////////////////////////////////////////////////////


HnetUniversePrint& HnetUniversePrint::operator=( HnetUniversePrint &universe)
{
	if (this != &universe)
		Copy( universe );

	return *this;
}


void HnetUniversePrint::Copy( HnetUniversePrint &universe)
{
	ObjPrint::Copy( universe ); 
	HnetUniverse::Copy ( universe );
	NodesNumbers::Copy( universe, m_isoTypeGraph.FirstNode(), universe.m_isoTypeGraph.FirstNode() );
}


IsoTypeNode HnetUniversePrint::CreateIsoType( int numOuterEntries, int numOuterExits )
{
	IsoTypeNode isoTypeNode = HnetUniverse::CreateIsoType( numOuterEntries, numOuterExits );
	NewIsoTypeID( isoTypeNode );
	return isoTypeNode;
}


IsoTypeNode HnetUniversePrint::CreateIsoTypeElementary( int numOuterEntries, int numOuterExits )
{
	IsoTypeNode isoTypeNode = HnetUniverse::CreateIsoTypeElementary( numOuterEntries, numOuterExits );
	NodeID id = NewIsoTypeID( isoTypeNode );
	return isoTypeNode;
}



IsoTypeNode HnetUniversePrint::CopyIsoType( const IsoTypeNode &isoTypeOrig )
{
	IsoTypeNode isoTypeNode = HnetUniverse::CopyIsoType( isoTypeOrig );
	NewIsoTypeID( isoTypeNode );
	return isoTypeNode;
}


void HnetUniversePrint::PrintIsoType( const IsoTypeNode &isoTypeNode ) 
{
	Hnet &hnet = GetHnet( isoTypeNode );
	HnetPrint hnetP( this, hnet );
	hnetP.Print("of isoType %d", GetIsoTypeID( isoTypeNode) );
}


void HnetUniversePrint::PrintIsoTypes( void ) const
{
	IsoTypeNode x = Root();
	printf( "\n Isotypes: " );
	while (x )
	{
		printf( "%u ", GetIsoTypeID( x ) );
		x = x.NextNode();
	}
	printf( "\n" );
}


//////////////////////////////////////////////////////////////////////
// private
//////////////////////////////////////////////////////////////////////

void HnetUniversePrint::PrintNeighbours( const Node &n ) const
{
	Edge e;

	printf( "  %04u %2u", m_isoTypeReferenceCounter[ n ], GetIsoTypeID( n ) );

	e = n.FirstOutgoingEdge();

	if ( e )
		printf( " ->" );

	while ( e )
	{
		printf( " %2u", GetIsoTypeID( e.TargetNode() ) );
		e = e.NextOutgoingEdge();
	}
}


void HnetUniversePrint::PrintBody( void ) const
{
	Edge e;
	Node n = m_isoTypeGraph.FirstNode();

	while ( n )
	{
		PrintNeighbours( n );
		printf( "\n" );
		n = n.NextNode();
	}
}



IsoTypeNode HnetUniversePrint::GetIsoTypeNode( NodeID nodeID )
{
	Node n = m_isoTypeGraph.FirstNode();

	while ( n )
	{
		if ( GetIsoTypeID( n ) == nodeID )
			return n;
		n = n.NextNode();
	}

	return Node(); // invalid node
}

