////////////////////////////////////////////////////////////////////////////////
// File         : SUVector.h
// Subsystem    : Graph
// Component    : graphs
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class SUVector
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 29.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//   
// This class implements a single user list of elements.
// It manages all memory allocation internaly.
// Instead of using STL no iterator must be handled.  
//     
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SUVECTOR_H__B8B59937_214F_4C61_B6D4_DEE642783C95__INCLUDED_)
#define AFX_SUVECTOR_H__B8B59937_214F_4C61_B6D4_DEE642783C95__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifdef WIN32
#pragma warning( disable : 4786 )  // Disable warning message C4786
#endif

#include <vector>

template< typename VectorElementType >
class SUVector  // single user list 
{
public:
	SUVector( const int numberOfElements = 0 ) : m_elements( numberOfElements ) {};
	SUVector( const SUVector &vec ) : m_it_current_element( vec.m_it_current_element ), m_elements( vec.m_elements ) {};
	virtual ~SUVector() {};
	SUVector& operator=(const SUVector &vec) {m_it_current_element = vec.m_it_current_element; m_elements = vec.m_elements; return *this;};
	VectorElementType& operator [](const int index);
	const VectorElementType& operator [](const int index) const;
	int Size( void ) const; 
	VectorElementType First( void ) const; 
	VectorElementType Last( void ) const; 
	VectorElementType Next( void ) const; 
	VectorElementType Prev( void ) const; 
	void AddLast( VectorElementType &elem ); 
	void DelLast( void ); 
	void DelAll( void ); 

private:
	mutable std::vector< VectorElementType >::iterator m_it_current_element;
	mutable std::vector< VectorElementType > m_elements;
};


// *********************** Template SUVector Attribute methods **********************************

template< typename VectorElementType >
VectorElementType &SUVector< VectorElementType >::operator [](const int index) 
{
	return m_elements[ index ];
}


template< typename VectorElementType >
const VectorElementType &SUVector< VectorElementType >::operator [](const int index) const
{
	return m_elements[ index ];
}


template< typename VectorElementType >
int SUVector< VectorElementType >::Size( void ) const
{
		return m_elements.size();
}


template< typename VectorElementType >
VectorElementType SUVector< VectorElementType >::First( void ) const
{
	m_it_current_element = m_elements.begin();

	if ( m_it_current_element == m_elements.end() )
		return VectorElementType();
	else 
		return *m_it_current_element;
}


template< typename VectorElementType >
VectorElementType SUVector< VectorElementType >::Last( void ) const
{
	if ( m_elements.size() == 0 )
		return VectorElementType();
	else 
	{
		m_it_current_element = m_elements.end();
		return *(--m_it_current_element);
	}
}

 
template< typename VectorElementType >
VectorElementType SUVector< VectorElementType >::Next( void ) const
{
	if ( ++m_it_current_element == m_elements.end() )
		return VectorElementType();
	else 
		return *m_it_current_element;
}

 
template< typename VectorElementType >
VectorElementType SUVector< VectorElementType >::Prev( void ) const
{
	if ( m_it_current_element == m_elements.begin() )
		return VectorElementType();
	else 
	{
		--m_it_current_element;
		return *m_it_current_element;
	}
}

 
template< typename VectorElementType >
void SUVector< VectorElementType >::AddLast( VectorElementType &node )
{
	m_elements.push_back( node );
}

 
template< typename VectorElementType >
void SUVector< VectorElementType >::DelLast( void )
{
	if( !m_elements.empty() )
		m_elements.pop_back();
}


template< typename VectorElementType >
void SUVector< VectorElementType >::DelAll( void )
{
	m_elements.clear();
}

#endif // !defined(AFX_SUVECTOR_H__B8B59937_214F_4C61_B6D4_DEE642783C95__INCLUDED_)
